{% extends "otree/FormPage.html" %}
{% load staticfiles otree_tags %}

{% comment %}
    this file used to be called Base.html
    If your app's templates contain this line:
    {% extends "global/Base.html" %}
    You should change it to:
    {% extends "global/Page.html" %}
{% endcomment %}


{% block global_styles  %}
{% endblock %}

{% block global_scripts  %}
{% endblock %}
