{% load i18n %}
<p>
    {% comment %}Translators: The text on the button the user clicks to get to the next page{% endcomment %}
        <button class="otree-next-button btn btn-primary next-button">{% blocktrans trimmed %}Next{% endblocktrans %}</button>
</p>
