{% extends "otree/admin/Session.html" %}
{% load floppyforms %}


{% block content %}
    {{ block.super }}
    <form action="" method="get" role="form" class="form" id="form">
        {% form form using %}
            {% formconfig row using "floppyforms/rows/bootstrap.html" %}
            <table cellpadding="10">
                <tr>
                    <td>{% formrow form.app_name %}</td>
                    <td>{% formrow form.round_number %}</td>
                    <td>
                        <button type="submit" id="submit" class="btn btn-primary">
                            Go
                        </button>
                    </td>
                </tr>
            </table>
        {% endform %}
    </form>

    {% include user_template %}

    {% if view.is_debug %}
        <br><br>
        {% include 'otree/includes/debug_info.html' %}
    {% endif %}

    <script>
    $(function() {
        $('#id_app_name').change(function() {
            $('#id_round_number').val('');
        });
    });
    </script>

{% endblock %}
