{% load staticfiles %}
<meta name="viewport" content="width=device-width, initial-scale=1.0">

<!-- compiled and minified CSS -->
<link rel="stylesheet" href="{% static 'bootstrap/css/bootstrap.min.css' %}">

<!-- Optional theme -->
<link rel="stylesheet" href="{% static 'bootstrap/css/bootstrap-theme.min.css' %}">

<link rel="stylesheet" href="{% static 'otree/css/theme.css' %}">

{# this actually belongs in internal_scripts, but we are in the process #}
{# of deprecating it, so some people's apps might still rely on jQuery being #}
{# available within the content block. #}
<script src="{% static 'otree/js/jquery.min.js' %}"></script>

<style>
    {% comment %}
    this is the long horizontal line below the page title;
    it seems many people want to get rid of it,
    and it looks strange when the title is left blank.
    also it will be gone in Bootstrap 4 I think.
    {% endcomment %}
    .page-header {
        border-bottom: none;
    }
</style>