{% load staticfiles %}
{% load i18n %}

<div id="otree-timeout" class="alert alert-warning" style="display:none;">
    <p>{{ timer_text }}
        <span style="font-weight: bold">
            <span class="glyphicon glyphicon-time"></span>
            <span id="minutes"></span>:<span id="seconds"></span>
        </span>
    </p>
</div>
